---
author: marc
title: Git
date: 9 novembre 2019
---

# Pourquoi versionner ?

---

![](./images/git/file_history.png){ width=60% }

Qu'est-ce que j'était en train de faire ?

Tout est cassé, comment revenir à une version fonctionelle ?

::: notes
Fichier/projet modifié il y a longtemps ou par d'autres personnes,
ou on revient dessus et on ne sait plus ce qu'il s'est passe,
ce qu'on était en train de faire

Le versionning peut nous aider a savoir sur quoi on était en train de travailler

Le versionning va surtout nous permettre de revenir à des versions fonctionnelles
(exemple : cas du projet où l'on rajoute une ligne qui casse tout)

:::

---

![](./images/git/collaborative_working.png)

J'ai écrit dans le même fichier que quelq'un d'autre

::: notes
Quand on travaille à plusieurs, mettre tout sur un dossier partagé n'est pas très pratique pour tous éditer des fichiers en même temps,
ni pour comprendre ce que les autres ont modifié.

Le versionnage va donc surtout être utilisé pour collaborer, et pouvoir partager des projets

:::

---

![](./images/git/psdrevisioning.jpg){ width=60% }

*Yash Bhardwaj*


::: notes
Le versionnage manuel des fichiers, c'est possible, mais ce n'est pas très efficace

On va voir un moyen de gérer le versionning plus facilement : git,
- le logiciel de versionnage le plus utilisé
:::



# Principe de git

---

## Commits

![](./images/git/commits.png)


::: notes
On va travailler avec une suite de ce qu'on appelle commit

Un commit correspond à l'état de tous les fichiers dans un dossier à un instant précis.

Dedans, on va stocker au minimum:
- la date
- l'auteur
- un message
- un hash, qui va identifier de manière unique ce commit, et sert à vérifier l'intégrité des fichiers

On peut donc stocker l'historique d'un dossier.

Mais qu'en est-il quand on veut expérimenter quelque chose que l'on voudrait pouvoir supprimer facilement,
partir dans deux directions différentes,
ou tout simplement travailler à deux sur des modifications du programme en même temps ?
:::

---

## Branches

![](./images/git/branch.png)


::: notes

Pour cela, on va utiliser des branches. Au lieu d'avoir juste une suite de commits, on va pouvoir partir dans deux directions différente à partir d'un même commit
on va créer une nouvelle branche et travailler dessus

Pareil quand on va travailler à plusieurs. On va chacun créer une branche pour ajouter la fonctionnalité sur laquelle on travaille.

:::


# Et en pratique alors ?

---


## Initialisation

```bash
# création du dossier
mkdir mon_dossier_versionne
cd mon_dossier_versionne

# initialisation
git init

# création de fichiers
touch dolphin
touch rabbit
```


::: notes

on va initialiser un répertoire git en allant dans le dossier qu'on veut versionner
et en utilisant git init

:::

---

## Staging

![](./images/git/staging_area.png){ width=40% }

```bash
# Voir l'état du dossier de travail
git status

# Ajouter un fichier à la zone de staging
git add dolphin

git status
```


::: notes

nos fichiers sont créés mais ne va pas être ajouté à la gestion de version

On va devoir rajouter les modifications que l'on veut versionner
dans la zone de staging avec add

Ici, la modification que l'on veut versionner va être l'ajout du fichier `dolphin`

:::

---

## Configurer

```bash
git config --global user.email mon.mail@la-bas.tf
git config --global user.name mon.pseudo
```

::: notes
dans les commits, on a besoin d'indiquer un nom ou un pseudo

On va donc devoir les configurer avec les commandes indiquées
(sans le global, cela configure juste pour le dossier en cours)
:::

---

## Versionner

![](./images/git/commits.png){ width=40% }

```bash
git commit
```

```vim
ajout d'un fichier vide
<ESC>:w
```

::: notes

Ici, on va réellement enregistrer notre version et noter ce qu'on a fait
:::

---

## Voir l'historique

```bash
git log
```

Quelles informations retrouve-t-on ?

::: notes

Pour voir l'historique des commit, la commande `log` est utilisée

:::

---

## Voir les modifications

```bash
# mettre du contenu dans le fichier rabbit
echo "Raoul" > rabbit

# voir la différence avec la zone de staging
git diff
```

Comment ajoutez-vous le fichier à la zone de staging ?

. . .

```bash
git add rabbit
git diff
```

::: notes
ajoutons du contenu dans le fichier rabbit

Avec `git diff` on va voir la différence avec la zone de staging
(et pas avec le dernier commit !)

Exemple : comment ajouter le fichier à la zone de staging ?
Et que se passe-t-il lors d'un nouveau lancement de git diff ?
:::

---

## Branches

```bash
git branch
```

Qu'observe-t'on ?

. . .

![](./images/git/head_master.png){ width=40% }

::: notes
On voit la liste des branches
La branche marquée d'une étoile est la branche courante

En fait, quand on dit qu'on est sur la branche master, ça veut dire qu'une référence (qu'on va appeler HEAD)
est placée sur le dernier commit de la branche master

Ce qu'on appelle HEAD va correspondre à l'état du dossier dans lequel on travaille
:::

---

```bash
# création d'une nouvelle branche
git branch feature/rabbit
```

Sur quelle branche est-on ?

. . .

```bash
# changer de branche
git checkout feature/rabbit
```

::: notes
On n'a pas changé de branche et on reste sur master

Pour changer de branche, on utilise checkout
:::

---

![](./images/git/head_feature_rabbit.png){ width=60% }

::: notes
checkout permet en fait de bouger la référence nommée HEAD,
Ici, on l'a déplacé sur le dernier commit de la branche feature/rabbit
:::

---

```bash
echo "Roseline" >> rabbit
git commit
```

Où est placée la référence HEAD ?

. . .

![](./images/git/head_feature_rabbit2.png){ width=40% }

---

## Fusion de branches

```bash
git checkout master
git merge feature/rabbit

git log
```

::: notes

on veut maintenant rajouter nos modifications dans la branche master
pour cela, on retoune sur la branche cible, et on utilise merge

:::

---

## Et quand il y a des conflits ?


```bash
# récupérer un exemple
git clone https://gitlab.clubelek.fr/formations/software-engineering/example-conflict.git
cd example-conflict

# découvrir la branche feature/rabbit
git checkout feature/rabbit
git checkout master

# Conflit lors de la fusion
git merge feature/rabbit
```

::: notes

si le même fichier a été modifié sur les deux branches, on ne pourra pas fusionner automatiquement

On va devoir gérer le conflit

Pour cela, on va voir dans le fichier où se situe le conflit
:::

---

```
Texte identique

<<<<<<< HEAD
Texte dans la branche sur laquelle HEAD est placé
=======
Texte
>>>>>>> feature/rabbit

Autre morceau de texte identique
```

Qu'est-ce que qu'on va changer si on veut garder Régine et Rolande ?


::: notes
Pour régler le conflit, on va modifier le fichier en conflit

Présentation du conflit

Il existe plein d'outils plus visuels que l'on ne va pas aborder
:::

## TL;DR

```bash
git init            # initialiser
git add <fichier>   # prendre compte les modifications
git commit          # enregistrer une version
git log             # voir les modifications
git branch <branch> # créer une branche
git branch          # voir sur quelle branche on est
git checkout <branch> # changer de branche
git merge <branch>  # fusionner des branches
```

::: notes
Voici un petit résumé de ce qu'on a vu jusqu'ici
:::


# Bonnes pratiques

---

## Branches

![](./images/git/git_branching_1.png){ width=35% }

---

![](./images/git/git_branching_2.png){ width=20% }

---

## Commits

- titre :
    - énoncé à l'impératif
    - moins de 50 caractères
- description
    - moins de 72 caractères
    - Pourquoi ? Qu'est-ce qui change ?

```git
add a rabbit

As a rabbit was missing, she have been added.
Her name is Régine
```

::: notes
Je ne préntends pas que ce commit est parfait, mais il permet
de comprendre ce qui a été fait et pour quelle raison.
:::

---

- commit uniquement quand ça marche
- en anglais

::: notes
Moins important dans l'immédiat, mais à adopter plus tard
:::

---

## Workflow

``` bash
# création d'une nouvelle branche et aller dessus
git checkout development
git checkout -b feature/add_rabbit

# implémenter la fonctionnalité

# mise à jour de la branche
git merge development

# Rajout des fonctionnalités dans development
git checkout development
git merge
```

# Astuces

---

Dans le fichier `${HOME}/.gitconfig`

```
[alias]
    st = status
    o = "log --pretty='format:%C(yellow)%h %Cblue%>(12)%ad %Cgreen%<(7)%aN%Cred%d %Creset%s' --date=relative --abbrev-commit"
```

::: notes
On peut configurer des alias
Le premier permet d'utiliser git status rapidement (commande essentielle)
Le deuxième permet d'avoir de jolis logs avec une seule ligne en utilisant `git o`
:::


---

## Découvrir le coupable

```bash
git blame <fichier>
```

::: note
Montre l'auteur de chaque ligne dans un fichier
Permet de savoir qui a introduit une feature, ou une erreur :derp:
:::

---

## Marquer

```bash
git tag v1.1.0
git tag poulet

git checkout v1.0.0
git checkout poulet
```

::: notes
pour marquer des versions, ou des commit particuliers, que l'on souhaite retrouver,
on va leur attribuer des tags

On peut ensuite y revenir en utilisant la commande checkout
:::

---

# TL;DR

---

```bash
git init            # initialiser
git add <fichier>   # prendre compte les modifications
git commit          # enregistrer une version
git log             # voir les modifications
git branch <branch> # créer une branche
git branch          # voir sur quelle branche on est
git checkout <branch>    # changer de branche
git checkout -b <branch> # créer une branche et aller dessus
git merge <branch>  # fusionner des branches
git blame <fichier> # regarder qui a ajouté une ligne
git tag <tagname>   # marquer un commit pour y revenir facilement
```
